package controller;

import javafx.application.Application;
import javafx.scene.*;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.ScrollEvent;
import javafx.stage.Stage;
import model.Board;
import view.*;

public class Main extends Application {
	@Override
	public void start(Stage stage) {

		// init --------------------------------------------------------------

		// init - some window settings
		stage.setMaximized(true); // maximized window
		stage.setTitle("Chess3D - Tuna Acikbas & Olivier Pillods"); // title

		// init - camera
		CustomCamera camera = new CustomCamera(1, 1000, 70); // create a camera and some settings

		// init - scene, main container
		Group group = new Group(); // will contain all the elements "to show" on scene
		CustomScene scene = new CustomScene(group, camera.getCamera(), SceneAntialiasing.BALANCED);

		// init - materials and shaders
		CustomMaterial material = new CustomMaterial(); // prepare all the 3D materials (shaders)

		// init - all lights
		CustomLight light = new CustomLight(); // setups different custom lights
		light.addAllToScene(group); // adds all the lights to scene

		// init - import all 3D models from .obj files
		Custom3dModel model = new Custom3dModel();

		// init - creates a wooden desk
		Desk desk = new Desk(material);
		desk.addAllToScene(group); // adds the desk to scene

		// init - creates a (model) list of all pieces
		Board board = new Board();

		// init - creates a (visual) chessboard of 8x8, and all (visual) 3d pieces
		BoardView boardView = new BoardView(model, material, board);
		boardView.addAllToScene(group); // adds the board and the pieces to scene

		// event --------------------------------------------------------------

		// event - camera rotation (mouse dragging)
		camera.initMouseControl(scene.get()); // allow user to move camera with mouse, with some constraints

		// event - camera zoom (mouse scroll)
		stage.addEventHandler(ScrollEvent.SCROLL, event -> camera.scrollCamera(event.getDeltaY()));

		// event - click on (visual) pieces to show (visually) available movements. (asked from model rules)
		for(int n = 0 ; n < 64 ; n++) {
			PieceView piece = boardView.getPiece(n);
			if(piece != null) {
				piece.getObj().addEventHandler(MouseEvent.MOUSE_CLICKED, event -> boardView.clickPiece(material, piece, board));
			}
		}

		// event - click on (visual) "colored tiles" to move the selected piece there
		for(int n = 0 ; n < 64 ; n++) {
			TileView tile = boardView.getTile(n);
			tile.getObj().addEventHandler(MouseEvent.MOUSE_CLICKED, event -> boardView.clickTile(material, model, tile, board));
		}

		// start --------------------------------------------------------------

		// start - links scene to application and shows application
		stage.setScene(scene.get());
		stage.show();
	}

	public static void main(String[] args) {
		launch();
	}
}